#ifndef HASH_TREE_H 
#define HASH_TREE_H 1

#include "itemset.hpp"
#include <boost/array.hpp>
#include <boost/foreach.hpp>

namespace A_PRIORI{

  enum {TABLE_SIZE = 10, BUCKET_SIZE = 10};
  enum nodetype{ Internal = 0, Leaf = 1 };

  typedef struct hashnode HashNode;
    
  struct hashnode{
    
    nodetype type;
    
    union{
      
      Transaction * leaf;               /* frequent itemset */
      HashNode * internal[TABLE_SIZE];  /*   internal node  */ 
    } node;
  }; // struct hashnode    

  class HashTree{

  private:
    HashNode * root;

  public:
    HashTree();
    ~HashTree();

    // 
    HashNode * get_root() const { return root;}
    
    //
    // the hash function compute an hash value for the ith item of the itemset
    //
    unsigned int hash(Itemset& itemset, unsigned int ith);

    //
    // Create a new node for the hashtree
    //
    HashNode * newnode(nodetype type);

    //
    // Delete a node from the hashtree
    //
    void deletenode(HashNode * n);

    //
    // Dump a subtree from the hashtree
    //
    void dump(HashNode * n);

    //
    //  Insert a new node in the hash three (this work recursively)
    //
    void insert(HashNode *n, Itemset& s, unsigned int ith);


  };  // class HashTree

}; // namespace
#endif
